/* filename:    bivgibb1.g
** description: The program generates draws from the posterior
**              distribution via a Metropolis-within-Gibbs Sampler
**              
** created:     02/21/02
** modified:   
*/

new;
closeall;
library user, pgraph;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\mispol\gauss-ds\pathspec.g
#include c:\projects\active\mispol\gauss-ds\bivspec.g

/*************************************** 
**  Configure the MCMC Algorithm
*/
mhrun   = "1";
nblocks = 100;
nsim    = 1000;
opath = parapath $+ "\\mhrun" $+ mhrun;

npsipara = (nvar-1)*(1+nvar*nlags);
nsigpara = nvar*(nvar+1)/2;
k        = 1+nlags*nvar;

/* Additional specifications for Metropolis Step
*/

_bounds =
    (1E-5 ~ 10  )|
    (1E-5 ~ 10  )|
    (1E-6 ~ 0.9999999 )|
    (-1   ~ 10 )|
    (-1   ~ 10 )|
    (1E-6 ~ 10 )|
    (1E-6 ~ 10  )|
    (1E-5 ~ 100 )|
    (1E-6 ~ 0.9999999 )|
    (1E-6 ~ 0.9999999 )|
    (1E-7 ~ 500     )|
    (1E-7 ~ 500     )|
    (1E-7 ~ 500     );

/******************************************************** 
** Import data
*/
#include c:\projects\active\mispol\gauss-ds\loaddata.g


/******************************************************** 
** Import the information for the prior density
*/
priorfile = lmodel $+ lprior $+ dataselstr $+ "pri.out";
load path = ^priopath prior[npara+1,3] = ^priorfile;

pmean  = prior[1:npara,1];
pstdd  = prior[1:npara,2];
pshape = prior[1:npara,3];
pshape = pshape.*para_maskinv[.,mspec];

/******************************************************** 
** Load Posterior Mode Estimates
*/
lpara = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ "pm";
open fhpara = ^lpara for read;
para = readr(fhpara,npara);
closeall fhpara;
load path = ^priopath para_names;

"Model            " $lmodel;
"Data set         " $dataselstr;
"Prior            " $lprior;
"Lambda           " $lamstr;
"nlags            " $nlagstr;
"MHrun Index      " $mhrun;
"";
"===================================";
"";
"";
"Parameter   | Estimate ";
outmat = para_names'~para;
let mask[1,2] = 0 1;
let fmt[2,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4;
d = printfm(outmat,(0 ~ 1),fmt);
"";
"Press Key to Continue";
kenter = keyw;
cls;

/******************************************************** 
** Load multiplication matrix for parameters
*/
lmult = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ "mu";
open fhmult = ^lmult for read;
SIGSCALE   = readr( fhmult,npara );
closeall fhmult;

lhess = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ "hs";
open fhhess = ^lhess for read;
SIGPROPINV  = readr( fhhess,npara );
closeall fhhess;

SIGPROPdim = sumc(para_maskinv[.,mspec]);
{u , s, v} = svd1(SIGPROPINV);
SIGPROPlndet = 0;

i = 1;
do until i > npara;
   if i <= SIGPROPdim;
      s[i,i]    = 1/s[i,i];
      SIGPROPlndet = SIGPROPlndet + ln(s[i,i]);
   endif;
   i = i+1;
endo;

/******************************************************** 
** Initialize Output files
*/
ostat    = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr $+ nlagstr $+ "st";
opara    = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr $+ nlagstr $+ "t";
opsipara = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr $+ nlagstr $+ "p";
osigpara = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr $+ nlagstr $+ "s";

statname = { POST LIKE REJ1 REJ2 };

create fhstat    =^ostat  with ^statname, 4, 8;
create fhpara    =^opara with ^para_names, npara, 8;
create fhpsipara =^opsipara with PSI, npsipara, 8;
create fhsigpara =^osigpara with SIG, nsigpara, 8;

/******************************************************** 
**   Metropolis-within-Gibbs Algorithm
*/

/* Some Preliminary Calculations
*/
SIG_ols = (YYYY-XXYY'*inv(XXXX)*XXYY)/nobs;
PSI_ols = inv(XXXX)*XXYY[.,2:nvar];

/* Initialize para_old
*/
valid = 0;
do until valid == 1;

   para_old   = para + cc0th*(sigscale*rndn(npara,1));
   para_old   = para_old.*para_maskinv[.,mspec] + para_fix[.,mspec].*para_mask[.,mspec];

   {postold, likeold} = fcn(vec(PSI_OLS),SIG_ols,para_old);

   if postold > -10000;
      valid = 1;
   endif;

endo;   

/* Initialize Sig_old
*/
{yyyyd, xxyyd, xxxxd, retcode} = varprio(para_old);

yyyyc = yyyy + lamsel*nobs*yyyyd;
xxyyc = xxyy + lamsel*nobs*xxyyd;
xxxxc = xxxx + lamsel*nobs*xxxxd;
nobsc = nobs + nobs*lamsel;    

SIG_tilde = ( YYYYc -XXYYc'*inv(XXXXc)*XXYYc )/nobsc;

H = nobsc*Sig_tilde;
Z = chol(inv(H))'*rndn(nvar,nobsc-k);
Siginv_old = Z*Z';
Sig_old    = inv(Z*Z');

/* Initialize PSI_old
*/
{B1, B2} = endoreg(para_old);
xxyytil  = xxyyc-xxxxc*B1;

PSIstar    = inv(xxxxc)*xxyytil*Siginv_old*B2'*inv( B2*Siginv_old*B2' );
PSIcovar   = inv( (B2*Siginv_old*B2').*.xxxxc );
PSIvec_old = vec(PSIstar) + cholpsd(PSIcovar)*rndn((nvar-1)*k,1);
PSI_old    = reshape(PSIvec_old,nvar-1,k)';

{postold, likeold} = fcn(PSIvec_old,SIG_old,para_old);

iblock = 1;
do until iblock > nblocks;

   tstart = date;

   parasim    = zeros(nsim,npara);
   psiparasim = zeros(nsim,npsipara);
   sigparasim = zeros(nsim,nsigpara);
   rejsim     = zeros(nsim,2);
   likesim    = zeros(nsim,1);
   postsim    = zeros(nsim,1);

   j=1;

   do until j > nsim;

      /* Draw from Psi | Sig, theta
      */
      
      {yyyyd, xxyyd, xxxxd, retcode} = varprio(para_old);

      yyyyc = yyyy + lamsel*nobs*yyyyd;
      xxyyc = xxyy + lamsel*nobs*xxyyd;
      xxxxc = xxxx + lamsel*nobs*xxxxd;
      nobsc = nobs + nobs*lamsel;    

      {B1, B2} = endoreg(para_old);
      xxyytilc = xxyyc - xxxxc*B1;
      yyyytilc = yyyyc - xxyyc'*B1 - B1'*xxyyc + B1'*xxxxc*B1;
      
      PSIstar  = inv(xxxxc)*xxyytilc*Siginv_old*B2'*inv( B2*Siginv_old*B2' );
      PSIcovar = inv( (B2*Siginv_old*B2').*.xxxxc );

      PSIvec_new = vec(PSIstar) + cholpsd(PSIcovar)*rndn((nvar-1)*k,1);
      PSI_new  = reshape(PSIvec_new,nvar-1,k)';

      psiparasim[j,.] = PSIvec_new';
      Psi_old         = Psi_new;      
      Psivec_old      = Psivec_new;

      PSI_prior = inv(xxxxd)*xxyyd[.,2:nvar];

      /*
      PHIprior0 = inv(xxxxd)*xxyyd; 
      PHIprior1 = B1+PSI_prior*B2;
      PHIprior0;
      PHIprior1;
      para_old;
      end;
      */

      /* Draw from Sig | Psi, theta
      */

      /* Draw from Proposal Density
      */
      Stilde = yyyytilc - B2'*PSI_new'*xxyytilc - xxyytilc'*PSI_new*B2 + B2'*PSI_new'*xxxxc*PSI_new*B2;
      Z = chol(inv(Stilde*ccsig))'*rndn(nvar,nobsc*ccsig);
      Siginv_new = Z*Z';
      Sig_new    = inv(Z*Z');

      {postnew, likenew} = fcn(PSIvec_new,Sig_new,para_old);
      {postold, likeold} = fcn(PSIvec_new,Sig_old,para_old);

      propdens_old = lpdfiw( Sig_old, Stilde*ccsig, nobsc*ccsig);
      propdens_new = lpdfiw( Sig_new, Stilde*ccsig, nobsc*ccsig);

      /* Decide whether to accept or reject the proposed draw
      */

      r = minc(1 | exp( postnew - propdens_new - postold + propdens_old ));

      if rndu(1,1) < r;
      
         /* Accept proposed jump
         */
         sigparasim[j,.] = vech(SIG_new)'; 
         Sig_old      = Sig_new;
         Siginv_old   = Siginv_new;

      else;

         /* Reject proposed jump
         */
         Sig_new      = Sig_old;
         Siginv_new   = Siginv_old;
         sigparasim[j,.] = vech(SIG_old)'; 
                
         rejsim[j,1] = 1;
     
      endif;

      /* Draw from theta | Psi, Sig
      */
      para_new   = para_old + ccth*(sigscale*rndn(npara,1));
      para_new   = para_new.*para_maskinv[.,mspec] + para_fix[.,mspec].*para_mask[.,mspec];

      {postnew, likenew} = fcn(PSIvec_new,Sig_new,para_new);
      {postold, likeold} = fcn(PSIvec_new,Sig_new,para_old);

      r = minc(1 | exp( postnew - postold));

      if rndu(1,1) < r;
      
         /* Accept proposed jump
         */
         postsim[j,1] = postnew;
         likesim[j,1] = likenew;
         parasim[j,.] = para_new';

         postold      = postnew;
         likeold      = likenew;
         para_old     = para_new;

      else;

         /* Reject proposed jump
         */
         likesim[j,1] = likeold;
         postsim[j,1] = postold;
         parasim[j,.] = para_old';
         para_new = para_old;
                
         rejsim[j,2] = 1;
     
      endif;
      
      /*
      locate 1,1;
      "Block" iblock "of" nblocks;
      "Simulation step" j "of" nsim;
      "Parameters     "; parasim[j,.]; 
      "Rejection      "; meanc(rejsim[1:j,.])';
      "Sig_OLS        "; sqrt(diag(SIG_ols))';
      "SIG_new        "; sqrt(diag(sig_new))';
      */      

      j = j+1;
     
   endo;


   locate 1,1;
   "Block" iblock "of" nblocks;
   "Rejection: perct "  meanc(rejsim)';
   "Parameters       "; meanc(parasim)'; 

   wr = writer(fhstat,postsim~likesim~rejsim);
   wr = writer(fhpara,parasim);
   wr = writer(fhpsipara,psiparasim);
   wr = writer(fhsigpara,sigparasim);

   tend = date;
   "Elapsed Time"; etstr(ethsec(tstart,tend));

   iblock = iblock+1;
endo;  

closeall fhstat, fhpara, fhpsipara, fhsigpara;
end;

/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (2) = fcn(PSIvec,SIG,para);
local lnpY, lnpriopsi, lnpriosig, lnpriopara, obsmean, obsvar, 
      lnpost, retcode, parabd_ind1, parabd_ind2;
 
    parabd_ind1 = para .> _bounds[.,1];
    parabd_ind2 = para .< _bounds[.,2];
    parabd_ind1 = delif(parabd_ind1,para_mask[.,mspec]);
    parabd_ind2 = delif(parabd_ind2,para_mask[.,mspec]);

    if (parabd_ind1 == 1) and (parabd_ind2 == 1);

       lnpy       = rflh(YYYY,XXXX,XXYY,Psivec,Sig,para);
       lnpriopsi  = priopsi(Psivec,Sig,para);
       lnpriosig  = priosig(Sig,para);
       lnpriopara = priopara(para,pmean,pstdd,pshape);

       lnpost  = lnpy + lnpriopsi + lnpriosig + lnpriopara;

    else;
      
       lnpost = -1E10;
       lnpy   = -1E10;

    endif;
    
retp(lnpost,lnpy);  
endp;

#include c:\projects\active\mispol\gauss-ds\dsgemod.src;
#include c:\projects\active\mispol\gauss-ds\bivobj.src;
#include c:\projects\active\mispol\gauss-ds\prioproc.src
