/* filename:    bivpol1.g
** description: the program converts posterior parameter draws into policy losses
**              scenario 1: psid and sigd do not change
** created:     11/06/01
*/

new;
closeall;
library pgraph, user;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\mispol\gauss-ds\pathspec.g
#include c:\projects\active\mispol\gauss-ds\bivspec.g
#include c:\projects\active\mispol\gauss-ds\loaddata.g

load path=^datapath nobs;
/********************************************************
** Define some Constants
*/
nread    = 10000;   /* block size */
nuse     = 100;
mhrun    = "2";
npsipara = (nvar-1)*(1+nvar*nlags);
nsigpara = nvar*(nvar+1)/2;
k        = 1+nlags*nvar;

lamstr_grid = " 5" $| "10" $| "50" ;
lamstr_n = rows(lamstr_grid);

lamstr_ind = 1;
do until lamstr_ind > lamstr_n;
lamstr1 = lamstr_grid[lamstr_ind];


/********************************************************
** Open files with Parameter Draws
*/
lpath  = parapath $+ "\\mhrun" $+ mhrun;  

lpara = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ "t";
open fhtpara = ^lpara for read;

lpara = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ "p";
open fhppara = ^lpara for read;

lpara = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ "s";
open fhspara = ^lpara for read;


/********************************************************
** Processing
*/ 

/******************************
** Loop over policy parameters
*/

psi1_ind = 1;
do until psi1_ind > psi1_n;
psi1 = psi1_grid[psi1_ind];

psi2_ind = 1;
do until psi2_ind > psi2_n;
psi2 = psi2_grid[psi2_ind];

rhor_ind = 1;
do until rhor_ind > rhor_n;
rhor = rhor_grid[rhor_ind];

     /********************************************************
     ** Initialize Output files
     */
     gridstr = ftos(100*psi1,"%*.*lf",3,0) $+ ftos(100*psi2,"%*.*lf",3,0) 
               $+ ftos(100*rhor,"%*.*lf",3,0);
     
     opath = parapath $+ "\\mhrun" $+ mhrun;

     opara = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ "p1" $+ gridstr;
     create fhpsipara = ^opara with PSI,npsipara , 8;

     opara = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ "s1" $+ gridstr;
     create fhsigpara = ^opara with SIGMA, nsigpara, 8;

     opara = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ "t1" $+ gridstr;
     create fhthpara = ^opara with THETA, npara, 8;

     opath = resupath $+ "\\mhrun" $+ mhrun;

     opara = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ "l1" $+ gridstr;
     create fhloss = ^opara with LOSS, 1+nvar, 8;

     /*********************************************************
     ** reset input files
     */
     drawrow = seekr( fhtpara, 1);
     drawrow = seekr( fhspara, 1);
     drawrow = seekr( fhppara, 1);
     
     eofloop = 0;
     loopct  = 1;

     locate 1,1;
     "Grid Point" psi1 psi2 rhor $lamstr1;

     /*********************************************************
     ** loop over draws
     */

     do until eofloop; 

        /* Read blocks of size nsim
        */
        psiparasim = readr( fhppara, nread );
        sigparasim = readr( fhspara, nread );
        thparasim  = readr( fhtpara, nread);

        indseq = seqa(1,1,nread);
        indseq = indseq % nuse;
        indseq = indseq .NE 0;

        psiparasim = delif(psiparasim,indseq);
        sigparasim = delif(sigparasim,indseq);
        thparasim  = delif(thparasim,indseq);

        nsim   = rows(thparasim);
        
        polpsiparasim = zeros(nsim,npsipara);
        polsigparasim = zeros(nsim,nsigpara );
        polthparasim  = zeros(nsim,npara);
        pollosssim    = zeros(nsim,1+nvar);
               
        locate 3,1;
        "Draw Loop   " loopct;
        "Blocksize   " nsim;
        eofloop = eof(fhtpara);
      
        j = 1;
        do until j > rows(thparasim);
                   
           /* calculate posterior draw of PSIdiff and SIGtrdiff
           */
           {yyyyd, xxyyd, xxxxd, retcode} = varprio(thparasim[j,.]');          

           PHIstar = inv(xxxxd)*xxyyd;
           PSIstar = PHIstar[.,2:nvar];          

           PSIdraw = reshape(psiparasim[j,.]',(nvar-1),k)';
           PSIdiff = PSIdraw - PSIstar;

           SIGstar = yyyyd - xxyyd'*inv(xxxxd)*xxyyd;
           SIGtrstar = chol(SIGstar)';

           SIGdraw = xpnd(sigparasim[j,.]');
           SIGtrdraw = chol(SIGdraw)';
           SIGtrdiff = SIGtrdraw - SIGtrstar;

           /* replace policy parameters
           */
           polthparasim[j,.] = psi1 ~ psi2  ~ rhor ~ thparasim[j,4:npara];
           
           /* calculate PSIstarpol SIGtrstarpol
           */
           {yyyyd, xxyyd, xxxxd, retcode} = varprio(polthparasim[j,.]');          

           PHIstarpol = inv(xxxxd)*xxyyd;
           PSIstarpol = PHIstarpol[.,2:nvar];          
           
           SIGstarpol = yyyyd - xxyyd'*inv(xxxxd)*xxyyd;
           SIGtrstarpol = chol(SIGstarpol)';

           /* calculate PSIpol SIGpol
           */
           PSIpol = PSIstarpol + PSIdiff;
           SIGpol = (SIGtrstarpol + SIGtrdiff)*(SIGtrstarpol + SIGtrdiff)';
           
           polpsiparasim[j,.] = vec(PSIpol)'; 
           polsigparasim[j,.] = vech(SIGpol)';
           
           /* calculate  PHIpol
           */
           {B1, B2} = endoreg(polthparasim[j,.]');
           PHIpol = B1 + PSIpol*B2;
           PHItpol = PHIpol';
            
           /* Calculate policy loss as fcn of unconditional variance 
           ** stable parameters only
           */
           if nlags == 1;
              PHItilde = PHItpol[.,1+1:1+nlags*nvar];
              SIGtilde = SIGpol;
           else;
              PHItilde = PHItpol[.,1+1:1+nlags*nvar] |
                         (eye(nvar*(nlags-1)) ~ zeros(nvar*(nlags-1),nvar));   
              SIGtilde = (SIGpol ~ zeros(nvar, nvar*(nlags-1)) )|
                         zeros(nvar*(nlags-1),nvar*nlags);              
           endif;
           
           if l_horizon == 0;
 
              /* unconditional variance provided VAR is stable
              */
              isstable = maxc(abs(eig(PHItilde))) < 1;
           
              if isstable == 1;
                 GAMM0pol = inv(eye(rows(PHItilde)^2) - (PHItilde.*.PHItilde) )*vec(SIGtilde);
                 GAMM0pol = reshape( GAMM0pol,rows(PHItilde),rows(PHItilde) );
                 pollosssim[j,1] = minc( sumc(diag(l_w*GAMM0pol[1:nvar,1:nvar])) | l_bound );
                 pollosssim[j,2:nvar+1] = minc( diag(GAMM0pol[1:nvar,1:nvar])' | ( ones(1,nvar)*l_bound ) )';
              else;
                 pollosssim[j,.] = ones(1,nvar+1)*l_bound;
              endif;

           else;

              /* discounted sum of conditional variances
              */
              GAMM0cond = SIGtilde;
              GAMM0pol  = l_bet*GAMM0cond;
              bet_sum = l_bet;
              polloss = sumc(diag(l_w*GAMM0pol[1:nvar,1:nvar]))/bet_sum;
              polloss1 = polloss;
              terminate = 0;
              jj = 2;
              
              do until terminate == 1;
              
                 GAMM0cond = PHItilde*GAMM0cond*PHItilde'+SIGtilde;
                 GAMM0pol = GAMM0pol + l_bet^jj*GAMM0cond;
                 bet_sum = bet_sum + l_bet^jj;
                 polloss = sumc(diag(l_w*GAMM0pol[1:nvar,1:nvar]))/bet_sum;
                 
                 if polloss > l_bound;
                    polloss = l_bound;
                    terminate = 1;
                 elseif abs(polloss-polloss1) < l_crit;
                    terminate = 1;
                 endif;
      
                 polloss1 = polloss;
                 jj = jj+1;
                 
              endo;
              pollosssim[j,1] = polloss;
              pollosssim[j,2:nvar+1] = diag( GAMM0pol[1:nvar,1:nvar] )'/bet_sum;

           endif;
           
           j = j+1;
           
        endo;

        /* Write Output
        */

        /*
        wr = writer(fhpsipara,polpsiparasim);
        wr = writer(fhsigpara,polsigparasim);
        wr = writer(fhthpara,polthparasim);
        */
        
        wr = writer(fhloss,pollosssim);

        loopct = loopct + 1;

        "Valid Losses [percent]:" 100*sumc(pollosssim[.,1] .< l_bound)/nsim;
        "Average Loss           " meanc(pollosssim);

     endo;

     fhpsipara = close(fhpsipara);
     fhsigpara = close(fhsigpara);
     fhthpara  = close(fhthpara);
     fhloss    = close(fhloss);

rhor_ind = rhor_ind + 1;
endo;

psi2_ind = psi2_ind + 1;
endo;

psi1_ind = psi1_ind + 1;
endo;

closeall fhppara, fhspara, fhtpara;

lamstr_ind = lamstr_ind+1;

endo;

#include c:\projects\active\mispol\gauss-ds\prioproc.src
#include c:\projects\active\mispol\gauss-ds\dsgemod.src
