/* filename:    bivpol3.g
** description: the program converts posterior parameter draws into policy losses
**              scenario 3: psid and sigd are re-drawn from the posterior distribution
** created:     11/06/01
*/

new;
closeall;
library pgraph, user;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\mispol\gauss-ds\pathspec.g
#include c:\projects\active\mispol\gauss-ds\bivspec.g
#include c:\projects\active\mispol\gauss-ds\loaddata.g

load path=^datapath nobs;
/********************************************************
** Define some Constants
*/
nread    = 1000;   /* block size */
nuse     = 100;
mhrun    = "1";
npsipara = (nvar-1)*(1+nvar*nlags);
nsigpara = nvar*(nvar+1)/2;
k        = 1+nlags*nvar;
nsimgibbs = 3;

lamstr_grid = " 5" $| "10" $| "50" ;
lamstr_n = rows(lamstr_grid);

lamstr_ind = 1;
do until lamstr_ind > lamstr_n;
lamstr1 = lamstr_grid[lamstr_ind];


/******************************************************** 
** Import data
*/
#include c:\projects\active\mispol\gauss-ds\loaddata.g

/******************************************************** 
** Import the information for the prior density
*/
priorfile = lmodel $+ lprior $+ dataselstr $+ "pri.out";
load path = ^priopath prior[npara+1,3] = ^priorfile;

pmean  = prior[1:npara,1];
pstdd  = prior[1:npara,2];
pshape = prior[1:npara,3];
pshape = pshape.*para_maskinv[.,mspec];


/********************************************************
** Open files with Parameter Draws
*/
lpath  = parapath $+ "\\mhrun" $+ mhrun;  

lpara = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ "t";
open fhtpara = ^lpara for read;

/********************************************************
** Processing
*/ 

/******************************
** Loop over policy parameters
*/

psi1_ind = 1;
do until psi1_ind > psi1_n;
psi1 = psi1_grid[psi1_ind];

psi2_ind = 1;
do until psi2_ind > psi2_n;
psi2 = psi2_grid[psi2_ind];

rhor_ind = 1;
do until rhor_ind > rhor_n;
rhor = rhor_grid[rhor_ind];

     /********************************************************
     ** Initialize Output files
     */
     gridstr = ftos(100*psi1,"%*.*lf",3,0) $+ ftos(100*psi2,"%*.*lf",3,0) 
               $+ ftos(100*rhor,"%*.*lf",3,0);
     
     opath = parapath $+ "\\mhrun" $+ mhrun;

     opara = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ "p3" $+ gridstr;
     create fhpsipara = ^opara with PSI,npsipara , 8;

     opara = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ "s3" $+ gridstr;
     create fhsigpara = ^opara with SIGMA, nsigpara, 8;

     opara = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ "t3" $+ gridstr;
     create fhthpara = ^opara with THETA, npara, 8;

     opath = resupath $+ "\\mhrun" $+ mhrun;

     opara = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ "l3" $+ gridstr;
     create fhloss = ^opara with LOSS, 1+nvar, 8;

     /*********************************************************
     ** reset input files
     */
     drawrow = seekr( fhtpara, 1);
     
     eofloop = 0;
     loopct  = 1;

     locate 1,1;
     "Grid Point" psi1 psi2 rhor $lamstr1;

     /*********************************************************
     ** loop over draws
     */

     do until eofloop; 

        /* Read blocks of size nsim
        */
        thparasim  = readr( fhtpara, nread);

        indseq = seqa(1,1,nread);
        indseq = indseq % nuse;
        indseq = indseq .NE 0;

        thparasim  = delif(thparasim,indseq);

        nsim   = rows(thparasim);
        
        polpsiparasim = zeros(nsim,npsipara);
        polsigparasim = zeros(nsim,nsigpara );
        polthparasim  = zeros(nsim,npara);
        pollosssim    = zeros(nsim,1+nvar);
               
        locate 3,1;
        "Draw Loop   " loopct;
        "Blocksize   " nsim;
        eofloop = eof(fhtpara);
      
        j = 1;
        do until j > rows(thparasim);
        
           /* replace policy parameters
           */
           THETAstar = (psi1 ~ psi2 ~ rhor ~ thparasim[j,4:npara])';
           polthparasim[j,.] = THETAstar';

           /* For each theta-para, draw a Psi and Sig from posterior using a 
           ** reduced run Gibbs sampler
           */
                     
           /* Initialize Sig_old
           */
           {yyyyd, xxyyd, xxxxd, retcode} = varprio(THETAstar);

           yyyyc = yyyy + lamsel*nobs*yyyyd;
           xxyyc = xxyy + lamsel*nobs*xxyyd;
           xxxxc = xxxx + lamsel*nobs*xxxxd;
           nobsc = nobs + nobs*lamsel;    

           {B1, B2} = endoreg(THETAstar);
           xxyytilc = xxyyc - xxxxc*B1;
           yyyytilc = yyyyc - xxyyc'*B1 - B1'*xxyyc + B1'*xxxxc*B1;

           SIG_tilde = ( YYYYc -XXYYc'*inv(XXXXc)*XXYYc )/nobsc;

           H = nobsc*Sig_tilde;
           Z = chol(inv(H))'*rndn(nvar,nobsc-k);
           Siginv_old = Z*Z';
           Sig_old    = inv(Z*Z');

           jj=1;

           do until jj > nsimgibbs;

              /* Draw from Psi | Sig, THETAstar
              */
      
              PSIbar   = inv(xxxxc)*xxyytilc*Siginv_old*B2'*inv( B2*Siginv_old*B2' );
              PSIcovar = inv( (B2*Siginv_old*B2').*.xxxxc );

              PSIvec_new = vec(PSIbar) + cholpsd(PSIcovar)*rndn((nvar-1)*k,1);
              PSI_new  = reshape(PSIvec_new,nvar-1,k)';

              /* Draw from Sig | Psi, THETAstar
              */

              /* Draw from Proposal Density
              */
              Stilde = yyyytilc - B2'*PSI_new'*xxyytilc - xxyytilc'*PSI_new*B2 + B2'*PSI_new'*xxxxc*PSI_new*B2;
              Z = chol(inv(Stilde))'*rndn(nvar,nobsc);
              Siginv_new = Z*Z';
              Sig_new    = inv(Z*Z');

              {postnew, likenew} = fcn(PSIvec_new,Sig_new,THETAstar);
              {postold, likeold} = fcn(PSIvec_new,Sig_old,THETAstar);

              propdens_old = lpdfiw( Sig_old, Stilde, nobsc);
              propdens_new = lpdfiw( Sig_new, Stilde, nobsc);

              /* Decide whether to accept or reject the proposed draw
              */

              r = minc(1 | exp( postnew - propdens_new - postold + propdens_old ));

              if rndu(1,1) < r;
      
                 /* Accept proposed jump
                 */
                 Sig_old      = Sig_new;
                 Siginv_old   = Siginv_new;
 
              else;

                 /* Reject proposed jump
                 */
                 Sig_new      = Sig_old;
                 Siginv_new   = Siginv_old;
     
              endif;
           
              jj = jj+1;
              
           endo;
           
           SIGpol = SIG_new;
           PSIpol = PSI_new;
           
           polpsiparasim[j,.] = vec(PSIpol)'; 
           polsigparasim[j,.] = vech(SIGpol)';
           
           /* calculate  PHIpol
           */
           PHIpol = B1 + PSIpol*B2;
           PHItpol = PHIpol';
            
           /* Calculate policy loss as fcn of unconditional variance
           ** stable parameters only
           */
           if nlags == 1;
              PHItilde = PHItpol[.,1+1:1+nlags*nvar];
              SIGtilde = SIGpol;
           else;
              PHItilde = PHItpol[.,1+1:1+nlags*nvar] |
                         (eye(nvar*(nlags-1)) ~ zeros(nvar*(nlags-1),nvar));   
              SIGtilde = (SIGpol ~ zeros(nvar, nvar*(nlags-1)) )|
                         zeros(nvar*(nlags-1),nvar*nlags);              
           endif;
           
           if l_horizon == 0;
 
              /* unconditional variance provided VAR is stable
              */
              isstable = maxc(abs(eig(PHItilde))) < 1;
           
              if isstable == 1;
                 GAMM0pol = inv(eye(rows(PHItilde)^2) - (PHItilde.*.PHItilde) )*vec(SIGtilde);
                 GAMM0pol = reshape( GAMM0pol,rows(PHItilde),rows(PHItilde) );
                 pollosssim[j,1] = minc( sumc(diag(l_w*GAMM0pol[1:nvar,1:nvar])) | l_bound);
                 pollosssim[j,2:nvar+1] = minc( diag(GAMM0pol[1:nvar,1:nvar])' | (ones(1,nvar)*l_bound))';
              else;
                 pollosssim[j,.] = ones(1,nvar+1)*l_bound;
              endif;

           else;

              /* discounted sum of conditional variances
              */
              GAMM0cond = SIGtilde;
              GAMM0pol  = l_bet*GAMM0cond;
              bet_sum = l_bet;
              polloss = sumc(diag(l_w*GAMM0pol[1:nvar,1:nvar]))/bet_sum;
              polloss1 = polloss;
              terminate = 0;
              jj = 2;
              
              do until terminate == 1;
              
                 GAMM0cond = PHItilde*GAMM0cond*PHItilde'+SIGtilde;
                 GAMM0pol = GAMM0pol + l_bet^jj*GAMM0cond;
                 bet_sum = bet_sum + l_bet^jj;
                 polloss = sumc(diag(l_w*GAMM0pol[1:nvar,1:nvar]))/bet_sum;
                 
                 if polloss > l_bound;
                    polloss = l_bound;
                    terminate = 1;
                 elseif abs(polloss-polloss1) < l_crit;
                    terminate = 1;
                 endif;
      
                 polloss1 = polloss;
                 jj = jj+1;
                 
              endo;
              pollosssim[j,1] = polloss;
              pollosssim[j,2:nvar+1] = diag(l_w*GAMM0pol[1:nvar,1:nvar])'/bet_sum;
           endif;
           
           j = j+1;
           
        endo;

        /* Write Output
        */
        /*
        wr = writer(fhpsipara,polpsiparasim);
        wr = writer(fhsigpara,polsigparasim);
        wr = writer(fhthpara,polthparasim);
        */
        wr = writer(fhloss,pollosssim);

        "Valid Losses [Percent]:" 100*sumc(pollosssim .NE l_invalid)/nsim;
        "Average Loss           " meanc(pollosssim);

        loopct = loopct + 1;

     endo;

     fhpsipara = close(fhpsipara);
     fhsigpara = close(fhsigpara);
     fhthpara  = close(fhthpara);
     fhloss    = close(fhloss);

rhor_ind = rhor_ind + 1;
endo;

psi2_ind = psi2_ind + 1;
endo;

psi1_ind = psi1_ind + 1;
endo;

closeall fhtpara;

lamstr_ind = lamstr_ind+1;

endo;


/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (2) = fcn(PSIvec,SIG,para);
local lnpY, lnpriopsi, lnpriosig, lnpriopara, obsmean, obsvar, 
      lnpost, retcode, parabd_ind1, parabd_ind2;
 
    lnpy       = rflh(YYYY,XXXX,XXYY,Psivec,Sig,para);
    lnpriopsi  = priopsi(Psivec,Sig,para);
    lnpriosig  = priosig(Sig,para);
    lnpriopara = priopara(para,pmean,pstdd,pshape);

    lnpost  = lnpy + lnpriopsi + lnpriosig + lnpriopara;
    
retp(lnpost,lnpy);  
endp;

#include c:\projects\active\mispol\gauss-ds\dsgemod.src;
#include c:\projects\active\mispol\gauss-ds\bivobj.src;
#include c:\projects\active\mispol\gauss-ds\prioproc.src
