dlibrary -a PszTgSen,PsdTgSen;

/* Define some constants
*/

para_names = { PSI1 PSI2 RHOR lnG lnPI lnrr KAP TAU RoG RoZ SigR SigG SigZ };
save path  = ^priopath para_names;
npara      = 13;

nvar   = 3;
T0     = 4;  /* number of observations to initialize lags of VAR */

nirf   = 10;
cumirf = ( 0 ~ 0 ~ 1 ); /* R, pi, d(y) */ 

/* Model Specification
**
** 1 = CB reacts to current inflation, current output,
** 
** para_mask is npara * (number of model specifications)
**   to fix a parameter under specification mspec, set the corresponding
**   element of para_mask to 1
** para_fix is npara * (number of model specifications)
**   and contains the values for the fixed parameters (all other entries
**   are irrelevant
*/

mspec = 1;
lmodel = ftos(mspec,"%*.*lf",1,0) ;

mprior = 1;
lprior = ftos(mprior,"%*.*lf",1,0);


para_mask    = zeros(npara,1);
para_maskinv = ones(npara,1) - para_mask;
para_fix     = zeros(npara,1);

/* Select lhs variable in endogenous regression
*/
y1sel = 1 ~ 0 ~ 0 ;

/* Selected data set
** 1 = US
*/
datasel    = 1;
dataselstr = ftos(datasel,"%*.*lf",1,0);

/* Use lamsel to select prior weight (multiples of actual sample)
*/
lamsel = 0.25;
lamstr = ftos(10*lamsel,"%*.*lf",2,0);

/* Use nlags to select number of lags in IV vector
*/
nlags = 2;
nlagstr = ftos(nlags,"%*.*lf",1,0);
 
/* Define grid for policy analysis
*/
psi1_min = 1.1;
psi1_max = 2.5;
psi1_n   = 11;

if psi1_n > 1;
   psi1_grid = seqa(psi1_min,(psi1_max-psi1_min)/(psi1_n-1),psi1_n);
else;
   psi1_grid = psi1_min;
endif;

psi2_min = 0.2;
psi2_max = 0.5;
psi2_n   = 1;

if psi2_n > 1;
   psi2_grid = seqa(psi2_min,(psi2_max-psi2_min)/(psi2_n-1),psi2_n);
else;
   psi2_grid = psi2_min;
endif;


rhor_min = 0.75;
rhor_max = 0.9;
rhor_n   = 1;

if rhor_n > 1;
   rhor_grid = seqa(rhor_min,(rhor_max-rhor_min)/(rhor_n-1),rhor_n);
else;
   rhor_grid = rhor_min;
endif;

/* weights in quadratic loss function
** y(t): Nom Rate, Inflation, Real GDP growth
*/
l_w = eye(3) .* (0 | 1 | 0.05)  ; /* mhrun2 */ 
l_w = eye(3) .* (1/4  | 1 | 1/4); /* mhrun1,3 */
l_horizon = 0;     /* l_horizon = 0 means unconditional variance */
l_bet     = 0.99;
l_invalid = -1000;
l_bound   = 20;
l_crit    = 1E-2;


/* Configuration of Metropolis Algorithm
*/
ccth  = 0.15;
cc0th = 0;
ccsig = 1;
