/* filename:    lossdif.g
** description: the program computes policy loss differentials
** created:     11/06/01
*/

new;
closeall;
library pgraph, user;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\mispol\gauss-ds\pathspec.g
#include c:\projects\active\mispol\gauss-ds\bivspec.g

/********************************************************
** Define some Constants
*/
nread    = 1000;   /* block size */
mhrun    = "1";

polstr_grid = "l0" $| "l1" $| "l2" $| "l3" $| "l1" $| "l2" $| "l3" $| "l1" $| "l2" $| "l3" ;
lamstr_grid = "na" $| " 5" $| " 5" $| " 5" $| "10" $| "10" $| "10" $| "50" $| "50" $| "50" ;

polstr_n = rows(polstr_grid);


/********************************************************
** Baseline loss
** nobaseline: if = 1, then use 0 as baseline
**
*/

nobaseline = 0;

psi10 = 1.90;
psi20 = 0.2;
rhor0 = 0.75;

psi10_ind = minindc( (psi1_grid - psi10)^2 );
psi20_ind = minindc( (psi2_grid - psi20)^2 );
rhor0_ind = minindc( (rhor_grid - rhor0)^2 );

psi10 = psi1_grid[psi10_ind];
psi20 = psi2_grid[psi20_ind];
rhor0 = rhor_grid[rhor0_ind];

/********************************************************
** Processing
*/ 

/******************************
** Loop over policy parameters
*/

polstr_ind = 1;
do until polstr_ind > polstr_n;
polstr1 = polstr_grid[polstr_ind];
lamstr1 = lamstr_grid[polstr_ind];

psi1_ind = 1;
do until psi1_ind > psi1_n;
psi1 = psi1_grid[psi1_ind];

psi2_ind = 1;
do until psi2_ind > psi2_n;
psi2 = psi2_grid[psi2_ind];

rhor_ind = 1;
do until rhor_ind > rhor_n;
rhor = rhor_grid[rhor_ind];

     /********************************************************
     ** Initialize Input / Output files
     */
     gridstr = ftos(100*psi1,"%*.*lf",3,0) $+ ftos(100*psi2,"%*.*lf",3,0) 
               $+ ftos(100*rhor,"%*.*lf",3,0);

     gridstr0 = ftos(100*psi10,"%*.*lf",3,0) $+ ftos(100*psi20,"%*.*lf",3,0) 
               $+ ftos(100*rhor0,"%*.*lf",3,0);
     
     lpath = resupath $+ "\\mhrun" $+ mhrun;
     opath = resupath $+ "\\mhrun" $+ mhrun;        

     lloss0 = lpath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ polstr1 $+ gridstr0;
     open fhloss0 = ^lloss0 for read;

     lloss = lpath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ polstr1 $+ gridstr;
     if NOT(lloss $== lloss0);
        open fhloss = ^lloss for read;
        drawrow = seekr( fhloss,1);
     endif;

     olossd = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ polstr1 $+ gridstr $+ "d";
     create fhlossd = ^olossd with LOSSD, 1+nvar, 8;
     
     eofloop = 0;
     loopct  = 1;

     locate 1,1;
     "Grid Point" psi1 psi2 rhor;

     /*********************************************************
     ** loop over draws
     */

     do until eofloop; 

        /* Read blocks of size nsim
        */
        losssim0  = readr( fhloss0, nread);

        if lloss $== lloss0;
           losssim = losssim0;
        else;
           losssim   = readr( fhloss, nread);
        endif;

        nsim      = rows(losssim0);
        
        invalidloss = (losssim0 .== l_invalid) .OR (losssim .== l_invalid);
        
        if nobaseline == 0;
           losssimdif = (invalidloss .NE 1).*(losssim-losssim0) + (invalidloss .== 1)*l_invalid;
        else;
           losssimdif = losssim;
        endif;

        locate 3,1;
        "Polstr      " polstr1;
        "Lamstr      " lamstr1;
        if nobaseline == 0;
           "Baseline  " psi10 psi20 rhor0;
        else;
           "No Baseline";
        endif;
        
        "Draw Loop   " loopct;
        "Blocksize   " nsim;

        eofloop = eof(fhloss0);
        loopct = loopct + 1;

        wr = writer(fhlossd,losssimdif);

     endo;
     
     if lloss $== lloss0;
        closeall fhlossd, fhloss0;
     else;
        closeall fhloss, fhloss0, fhlossd;
     endif;

rhor_ind = rhor_ind + 1;
endo;

psi2_ind = psi2_ind + 1;
endo;

psi1_ind = psi1_ind + 1;
endo;

polstr_ind = polstr_ind + 1;
endo;
