/* filename:    lossmom.g
** description: the program computes posterior means, stdd, and confidence
**              intervals for policy loss differentials
** created:     11/06/01
*/

new;
closeall;
library pgraph, user;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\mispol\gauss-ds\pathspec.g
#include c:\projects\active\mispol\gauss-ds\bivspec.g

/********************************************************
** Define some Constants
*/
nread    = 1000;   /* block size */
mhrun    = "1";
nburn    = 101;
hpdprob  = 0.9;

polstr_grid = "l0" $| "l1" $| "l2" $| "l3" $| "l1" $| "l2" $| "l3" $| "l1" $| "l2" $| "l3" ;
lamstr_grid = "na" $| " 5" $| " 5" $| " 5" $| "10" $| "10" $| "10" $| "50" $| "50" $| "50" ;

polstr_n = rows(polstr_grid);


/******************************
** Loop over policy parameters
*/

polstr_ind = 1;
do until polstr_ind > polstr_n;
polstr1 = polstr_grid[polstr_ind];
lamstr1 = lamstr_grid[polstr_ind];

psi1_ind = 1;
do until psi1_ind > psi1_n;
psi1 = psi1_grid[psi1_ind];

psi2_ind = 1;
do until psi2_ind > psi2_n;
psi2 = psi2_grid[psi2_ind];

rhor_ind = 1;
do until rhor_ind > rhor_n;
rhor = rhor_grid[rhor_ind];

     /********************************************************
     ** Initialize Input / Output files
     */
     gridstr = ftos(100*psi1,"%*.*lf",3,0) $+ ftos(100*psi2,"%*.*lf",3,0) 
               $+ ftos(100*rhor,"%*.*lf",3,0);
     
     lpath = resupath $+ "\\mhrun" $+ mhrun;
     opath = resupath $+ "\\mhrun" $+ mhrun;        

     lloss = lpath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ polstr1 $+ gridstr $+ "d";
     open fhloss = ^lloss for read;

     olossdm = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ polstr1 $+ gridstr $+ "dm";
     create fhlossdm = ^olossdm with MOM, 1+nvar, 8;
     
     eofloop = 0;
     loopct  = 1;

     "Grid Point" psi1 psi2 rhor;

     /*********************************************************
     ** loop over draws
     */

     drawrow   = seekr( fhloss, nburn); 
     drawcol   = readr( fhloss, 1);

     ndraws  = 1;
   
     do until eofloop; 

        drawblock = readr( fhloss, nread );
        drawcol   = drawcol | drawblock;
        ndraws    = ndraws + rows(drawblock);
        eofloop   = eof(fhloss);
         
     endo;
     
     /* Remove invalid draws
     */
     drawcol = delif(drawcol,drawcol.==l_invalid);
     ndraws_valid = rows(drawcol);

     drawci   = hpdint(drawcol,hpdprob);
     drawmean = meanc(drawcol);
     drawstdd = stdc(drawcol);
   
     "Polstr      " polstr1;
     "Lamstr      " lamstr1;
     "Mean Stdd CI" drawmean~drawstdd~drawci';
     "Fraction valid" 100*ndraws_valid/ndraws;
     "----------------------------------------";
     
     wr = writer(fhlossdm, (drawmean' | drawstdd' | drawci ));

     closeall fhloss, fhlossdm;

rhor_ind = rhor_ind + 1;
endo;

psi2_ind = psi2_ind + 1;
endo;

psi1_ind = psi1_ind + 1;
endo;

polstr_ind = polstr_ind + 1;
endo;



/**************************************************************/
/*                         Procedures                         */
/**************************************************************/

proc(1) = hpdint(draws,percent);
local     ndraws, drawdim, hpdband, nwidth, i, drawcoli,
          bup, minwidth, newwidth, j;

drawdim   = cols(draws);
ndraws    = rows(draws);
hpdband   = zeros(2,drawdim);
nwidth    = int(percent*ndraws);

i = 1;
do until i > drawdim;
   drawcoli = draws[.,i];
   /* sort response for period i, element 1 is max
   */
   drawcoli = rev(sortc(drawcoli,1));
   bup   = 1;
   minwidth  = drawcoli[1] - drawcoli[nwidth];
   j = 2;
   do until j > (ndraws-nwidth+1);
      newwidth = drawcoli[j] - drawcoli[j+nwidth-1];
      if newwidth < minwidth;
         bup = j;
         minwidth = newwidth;
      endif;
      j = j+1;
   endo;
   hpdband[2,i] = drawcoli[bup];
   hpdband[1,i] = drawcoli[bup+nwidth-1];
   i = i+1;
endo;
retp(hpdband);
endp;
