/****************************************************************
** 
** Create a multiple plots with posterior means and confidence intervals
** of loss differentials
*/                                                          

/* filename: lossplt3.g
** created:  09/27/01
*/
new;
library pgraph;
cls;

#include c:\projects\active\mispol\gauss-ds\pathspec.g
#include c:\projects\active\mispol\gauss-ds\bivspec.g

mhrun = "1";
losssel = 1;  /* 1 = loss, 2-nvar = univ var */


/* Define plot grid
** Fix 3 out of 4 dimensions
*/

psi10 = 1.9;
psi20 = 0.2;
rhor0 = 0.75;

psi10_ind = minindc( (psi1_grid - psi10)^2 );
psi20_ind = minindc( (psi2_grid - psi20)^2 );
rhor0_ind = minindc( (rhor_grid - rhor0)^2 );

psi10_grid = psi1_grid[psi10_ind];
psi20_grid = psi2_grid[psi20_ind];
rhor0_grid = rhor_grid[rhor0_ind];

psi10_n = 1;
psi20_n = 1;
rhor0_n = 1;

psi10_grid = psi1_grid;
psi10_n    = psi1_n;
loss_n     = psi1_n;
loss_grid  = psi1_grid;


/* Grid for policy experiments
*/
polstr_grid = "l0" $| "l2" $| "l2" $| "l2" $| "l1" $| "l1" $| "l1" $| "l3" $| "l3" $| "l3";
lamstr_grid = "na" $| "50" $| "10" $| " 5" $| "50" $| "10" $| " 5" $| "50" $| "10" $| " 5";

plot_mult = 1 | 3 | 3 | 3;
plot_n = rows(plot_mult);

/* Axis labels 
*/
legstr   = "Lam = 5.0\000Lam = 1.0\000Lam = 0.5";
titlestr = "Scenario 1" $| 
           "Scenario 2" $| 
           "Scenario 3" $| 
           "Scenario 4" ;
xstr     = "PSI 1";

/******************************
** Loop over policy parameters
*/

graphset;
begwind;
margin(0,0,0.2,0.2);
window(2,2,0);
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.35;
/*_pcolor = {2 4 3} ;*/
_pcolor = {2 2 2} ;
_pltype = {6 1 2};
_protate = 0; 
_plctrl = 0;
_plwidth = 5.5;
_psymsiz = 5;
_pline = ( 1 ~ 1 ~ 0 ~ 0 ~ 1 ~ 0 ~ 4 ~ 1 );
ytics(-2,8,1,1);

polstr_ind = 1;

plot_ind = 1;
do until plot_ind > plot_n;

loss_mat = zeros(loss_n,plot_mult[plot_ind]);

plot_lineind = 1;
do until plot_lineind > plot_mult[plot_ind];

polstr1 = polstr_grid[polstr_ind];
lamstr1 = lamstr_grid[polstr_ind];

grid_count = 1;

psi1_ind = 1;
do until psi1_ind > psi10_n;
psi1 = psi10_grid[psi1_ind];

psi2_ind = 1;
do until psi2_ind > psi20_n;
psi2 = psi20_grid[psi2_ind];

rhor_ind = 1;
do until rhor_ind > rhor0_n;
rhor = rhor0_grid[rhor_ind];

     gridstr = ftos(100*psi1,"%*.*lf",3,0) $+ ftos(100*psi2,"%*.*lf",3,0) 
               $+ ftos(100*rhor,"%*.*lf",3,0);
     
     lpath = resupath $+ "\\mhrun" $+ mhrun;

     lloss = lpath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr1 $+ nlagstr $+ polstr1 $+ gridstr $+ "dm";
     open fhloss = ^lloss for read;

     loss_summary = readr(fhloss,4);
 
     loss_mat[grid_count,plot_lineind] = loss_summary[1,losssel]; 

     fhloss = close(fhloss);

     grid_count = grid_count+1;

rhor_ind = rhor_ind + 1;
endo;

psi2_ind = psi2_ind + 1;
endo;

psi1_ind = psi1_ind + 1;
endo;

plot_lineind = plot_lineind + 1;
polstr_ind = polstr_ind + 1;

endo;

if plot_ind == 2;
   _plegstr = legstr;
   _plegctl = {2 4 5 3.3};
else;
   _plegctl = { };
endif;

xtics(1.0,2.5,0.5,2);
xlabel(xstr);
title(titlestr[plot_ind]);
xy(loss_grid,loss_mat);
nextwind;

plot_ind = plot_ind+1;
endo;

endwind;
