/* filename: loaddata.g
** description: reads time series data from ASCII data set into
** Gauss variable SERIES
** created:  12/10/01
**
*/

if datasel == 1;

  /* Time series, extracted 21/11/01
  ** Gross Domestic Product (SAAR, Bil.Chn.1996$), Bureau of Economic Analysis
  ** CPI-U: All Items (SA, 1982-84=100), Bureau of Labor Statistics
  ** Federal Funds Rate 1,2
  ** Quarterly Data from 1947:I to 2001:III
  */

  datafile = datapath $+ "//data4.txt";
  load data_q[219,4]=^datafile;

  /* Compute growth rates: from 1947:II to 2001:III, 218 observations
  */
  series =  100*( ln(data_q[2:rows(data_q),1])-ln(data_q[1:rows(data_q)-1,1]) ) ~
            400*( ln(data_q[2:rows(data_q),2])-ln(data_q[1:rows(data_q)-1,2]) ) ~
                ( data_q[2:rows(data_q),3] );
                
          
  /* Eliminate Observations for which the Federal Funds Rate is not available
  ** -> 189 observations, starting 1954:III
  */
  series = series[30:rows(series),.];

  nobs   = rows(series);
  ti     = seqa(1954.5,0.25,nobs);
  
  /* Eliminate additional observations: start 1982:I --> 79 observations
  */
  series = series[111:nobs,.];
  ti = ti[111:nobs];
  nobs = rows(series);
    

endif;

/* Change order of variables
** Nom Rate, Inflation, Real GDP growth
*/
series = series[.,3] ~ series[.,2] ~ series[.,1];

/* Create YY and XX matrices
*/
YY     = series[1+T0:nobs,.];
XX     = ones(nobs-T0, 1+nvar*nlags);
tivar  = ti[1+T0:nobs,.];

i = 1;
do until i > nlags;
   XX[.,2+(i-1)*nvar:1+i*nvar] = series[T0-i+1:nobs-i,.];
   i = i+1;
endo;

YYYY = YY'*YY;
XXYY = XX'*YY;
XXXX = XX'*XX;

nobs = rows(YY);

save path = ^datapath nobs, xxxx, yyyy, xxyy, yy , xx;
