/* filename:    bivdd2.g
** description: Computes numerical approximations to the data density
**              based on MCMC output
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

/******************************************************************
**         Load Parameter Draws from MH Output
*/
#include c:\projects\active\mispol\gauss-ds\pathspec.g;
#include c:\projects\active\mispol\gauss-ds\bivspec.g;

mhrun = "1";

lpath       = parapath $+ "\\mhrun" $+ mhrun;
lstat       = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr $+ nlagstr $+ "st";
lthpara     = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr $+ nlagstr $+ "t";
lpsipara    = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr $+ nlagstr $+ "p";
lsigpara    = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr $+ nlagstr $+ "s";

open fhthpara  = ^lthpara for read;
open fhpsipara = ^lpsipara for read;
open fhsigpara = ^lsigpara for read;
open fhstat    = ^lstat for read;

nburn   = 10001;
nblock  = 10000;
densfac = -250;

thpararow  = readr(fhthpara,nburn);
psipararow = readr(fhpsipara,nburn);
sigpararow = readr(fhsigpara,nburn);

nthpara   = sumc(para_maskinv[.,mspec]);
npsipara  = cols(psipararow);
nsigpara  = cols(sigpararow);

nallpara  = nthpara+npsipara+nsigpara;

thpararow  = seekr(fhthpara,nburn);
psipararow = seekr(fhpsipara,nburn);
sigpararow = seekr(fhsigpara,nburn);
statrow    = seekr(fhstat,nburn);

/* 
** Method:  modified harmonic mean
*/

p = seqa(0.1,0.1,9); 
pcrit = cdfchii(p,ones(rows(p),1)*nallpara);

/* Part 1: Compute posterior moments for parameters
*/

eofloop = 0;
ndraws  = 0;
sumpara  = zeros(nallpara,1);
sumpara2 = zeros(nallpara,nallpara);

do until eofloop; 

   thetasim = readr( fhthpara, nblock );
   thetasim = delif(thetasim',para_mask[.,mspec])';
   
   psisim = readr( fhpsipara, nblock);
   sigsim = readr( fhsigpara, nblock);
   
   parasim = thetasim~psisim~sigsim;

   sumpara  = sumpara + sumc(parasim) ;
   sumpara2 = sumpara2 + parasim'*parasim;
   ndraws  = ndraws + rows(parasim);

   eofloop = eof(fhthpara);

endo;

parahat      = sumpara / ndraws;
parasig      = sumpara2 / ndraws - parahat*parahat';
parasiginv   = inv(parasig);
parasiglndet = ln(det(parasig));


/* Part 2:  Compute Density estimate
*/

thpararow  = seekr(fhthpara,nburn);
psipararow = seekr(fhpsipara,nburn);
sigpararow = seekr(fhsigpara,nburn);
statrow    = seekr(fhstat,nburn);

eofloop = 0;
ndraws  = 0;

suminvlike = zeros(rows(p),1);

do until eofloop;

   statsim  = readr( fhstat, nblock);

   thetasim = readr( fhthpara, nblock );
   thetasim = delif(thetasim',para_mask[.,mspec])';
   
   psisim = readr( fhpsipara, nblock);
   sigsim = readr( fhsigpara, nblock);
   
   parasim = thetasim~psisim~sigsim;

   i = 1;
   do until i > rows(parasim);
    
      lnfpara = -ln(p) - 0.5*nallpara*ln(2*pi) - 0.5*parasiglndet
                 -0.5*(parasim[i,.] - parahat')*parasiginv*(parasim[i,.]' - parahat);
      indpara = ((parasim[i,.] - parahat')*parasiginv*(parasim[i,.]' - parahat) .< pcrit );
      invlike  = exp(lnfpara - statsim[i,1] + densfac).*indpara;

      suminvlike = suminvlike + invlike;
      i = i+1;
   endo;

   ndraws = ndraws + rows(parasim);
   locate 1,1;
   ndraws;
   eofloop = eof(fhthpara);

endo;
closeall fhthpara, fhpsipara, fhsigpara, fhstat;

meaninvlike = suminvlike/ndraws;

pym3        = densfac-ln(meaninvlike);

"Model:  " $lmodel;
"Prior:  " $lprior;
"MhRun:  " $mhrun;
"Dataset:" $dataselstr;
"======================";
"";

"Marginal Data Density" pym3;
"Average Across p     " meanc(pym3);

end;
