/* filename:    dsgeirf.g
** description: the program converts posterior parameter draws into identified 
**              impulse response functions
** created:     11/06/01
*/

new;
closeall;
library pgraph, user;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\mispol\gauss-ds\pathspec.g
#include c:\projects\active\mispol\gauss-ds\bivspec.g
#include c:\projects\active\mispol\gauss-ds\loaddata.g

load path=^datapath nobs;
/********************************************************
** Define some Constants
*/
nread    = 1000;   /* block size */
mhrun    = "1";
npsipara = (nvar-1)*(1+nvar*nlags);
nsigpara = nvar*(nvar+1)/2;
k        = 1+nlags*nvar;

/********************************************************
** Open files with Parameter Draws
**
** polstr: "" OR "0";
** 
*/
lpath  = parapath $+ "\\mhrun" $+ mhrun;  
polstr = "";

psi1 = 1.5;
psi2 = 0.2;
rhor = 0.7;

psi1_ind = minindc( (psi1_grid - psi1)^2 );
psi2_ind = minindc( (psi2_grid - psi2)^2 );
rhor_ind = minindc( (rhor_grid - rhor)^2 );

psi1 = psi1_grid[psi1_ind];
psi2 = psi2_grid[psi2_ind];
rhor = rhor_grid[rhor_ind];

gridstr = ftos(100*psi1,"%*.*lf",3,0) $+ ftos(100*psi2,"%*.*lf",3,0) 
          $+ ftos(100*rhor,"%*.*lf",3,0);

if polstr $== "";
   gridstr = "";
endif;

lpara = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ "na" $+ nlagstr $+ "t"
        $+ polstr $+ gridstr;
open fhtpara = ^lpara for read;


/********************************************************
** Initialize output file
*/ 
     
opath = resupath $+ "\\mhrun" $+ mhrun;

odsgeirf = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ "na" $+ nlagstr $+ "dsgeirf" 
         $+ polstr $+ gridstr;

create fhdsgeirf = ^odsgeirf with IRF, nirf*nvar^2 , 4;
     
eofloop = 0;
loopct  = 1;

/*********************************************************
** loop over draws
*/

do until eofloop; 

   /* Read blocks of size nsim
   */
   thparasim  = readr( fhtpara, nread);

   nsim   = rows(thparasim);
        
   dsgeirfsim = zeros(nsim,nirf*nvar^2);
                   
   locate 1,1;
   "Draw Loop   " loopct;
   "Blocksize   " nsim;
   eofloop = eof(fhtpara);
      
   j = 1;
   do until j > nsim;
                 
      dsgeirfsim[j,.] = vec(dsgeirf(thparasim[j,.]',nirf))';
      j = j+1;
      
   endo;
         
   /* Write Output
   */
   wr = writer(fhdsgeirf,dsgeirfsim);
   loopct = loopct + 1;

endo;

closeall fhdsgeirf;

#include c:\projects\active\mispol\gauss-ds\prioproc.src
#include c:\projects\active\mispol\gauss-ds\dsgemod.src
#include c:\projects\active\mispol\gauss-ds\bivirf.src
