/* filename:    dsgepol.g
** description: the program converts posterior parameter draws into policy losses
**              scenario 0: use DSGE model
** created:     11/06/01
*/

new;
closeall;
library pgraph, user;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\mispol\gauss-ds\pathspec.g
#include c:\projects\active\mispol\gauss-ds\bivspec.g
#include c:\projects\active\mispol\gauss-ds\loaddata.g

load path=^datapath nobs;
/********************************************************
** Define some Constants
*/
nread    = 1000;   /* block size */
nuse     = 10;
mhrun    = "2";
lamstr   = "na";
npsipara = (nvar-1)*(1+nvar*nlags);
nsigpara = nvar*(nvar+1)/2;
k        = 1+nlags*nvar;


/********************************************************
** Processing
*/ 

/******************************
** Loop over policy parameters
*/

psi1_ind = 1;
do until psi1_ind > psi1_n;
psi1 = psi1_grid[psi1_ind];

psi2_ind = 1;
do until psi2_ind > psi2_n;
psi2 = psi2_grid[psi2_ind];

rhor_ind = 1;
do until rhor_ind > rhor_n;
rhor = rhor_grid[rhor_ind];

     /********************************************************
     ** Initialize Output files
     */
     gridstr = ftos(100*psi1,"%*.*lf",3,0) $+ ftos(100*psi2,"%*.*lf",3,0) 
               $+ ftos(100*rhor,"%*.*lf",3,0);
  
     lpath  = parapath $+ "\\mhrun" $+ mhrun;  
     opath = parapath $+ "\\mhrun" $+ mhrun;        

     lpara = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr $+ nlagstr $+ "t";
     open fhtpara = ^lpara for read;

     opara = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr $+ nlagstr $+ "t0" $+ gridstr;
     create fhthpara = ^opara with THETA, npara, 8;

     opath = resupath $+ "\\mhrun" $+ mhrun;        

     opara = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ lamstr $+ nlagstr $+ "l0" $+ gridstr;
     create fhloss = ^opara with LOSS, 1, 8;

     /*********************************************************
     ** reset input files
     */
     drawrow = seekr( fhtpara, 1);
     
     eofloop = 0;
     loopct  = 1;

     locate 1,1;
     "Grid Point" psi1 psi2 rhor;

     /*********************************************************
     ** loop over draws
     */

     do until eofloop; 

        /* Read blocks of size nsim
        */
        thparasim  = readr( fhtpara, nread);

        indseq = seqa(1,1,nread);
        indseq = indseq % nuse;
        indseq = indseq .NE 0;
 
        thparasim = delif(thparasim,indseq);

        nsim   = rows(thparasim);
        
        polthparasim  = zeros(nsim,npara);
        pollosssim    = zeros(nsim,1);
               
        eofloop = eof(fhtpara);
      
        j = 1;
        do until j > nsim;
        
           /* replace policy parameters
           */
           polthparasim[j,.] = psi1 ~ psi2 ~ rhor ~ thparasim[j,4:npara];

           /* Evaluate loss
           */
           pollosssim[j,.] = dsgeloss(polthparasim[j,.]');
                    
           j = j+1;
           
        endo;

        /* Write Output
        */

        wr = writer(fhthpara,polthparasim); 
        wr = writer(fhloss,pollosssim);

        locate 3,1;
        "Draw Loop   " loopct;
        "Blocksize   " nsim;
        "Average Loss" meanc(pollosssim);


        loopct = loopct + 1;

     endo;

     closeall fhthpara, fhtpara, fhloss;

rhor_ind = rhor_ind + 1;
endo;

psi2_ind = psi2_ind + 1;
endo;

psi1_ind = psi1_ind + 1;
endo;


#include c:\projects\active\mispol\gauss-ds\prioproc.src
#include c:\projects\active\mispol\gauss-ds\dsgemod.src

/****************************************************************************/

proc (1) = dsgeloss(para);
local TT, TC, TEPS, TETA, RC,
      DD, RR, VV, HH, QQ, ZZ, nstate,
      GAM0s, GAM0fin, GAM0inf, Pt, j,
      polloss, polloss1, terminate, bet_sum;
      
/* solve the DSGE model
*/
{TT,TC,TEPS,TETA,RC} = dsgesolv(para);
nstate = cols(TT);

RR = TEPS;

/* create system matrices for state space model
*/

/* composition of measured vector
**  R, pi, d(y)
*/

ZZ      = zeros(nvar,nstate);
ZZ[3,1] = 1; ZZ[3,6] = 1; ZZ[3,9] = -1; 
ZZ[2,2] = 4; /* measured inflation is annualized */ 
ZZ[1,3] = 4; /* measured interest rates are annualized */

HH = zeros(nvar,nvar);
QQ = createcov(para[11:13,1]);
VV = zeros(nvar,nvar);

if l_horizon == 0; 

   GAM0s    = real(dlyap(TT,RR*QQ*RR'));
   GAM0inf  = ZZ*GAM0s*ZZ' + ZZ*RR*VV + (ZZ*RR*VV)' + HH;
   polloss  = l_vec'*GAM0inf*l_vec;
   polloss  = minc(l_bound,polloss);
   
else;

   Pt = RR*QQ*RR';
   GAM0fin = l_bet*( ZZ*Pt*ZZ' + ZZ*RR*VV + (ZZ*RR*VV)' + HH );
   bet_sum = l_bet;
   polloss = l_vec'*GAM0fin*l_vec/bet_sum;
   polloss1 = polloss;
   terminate = 0;
   j = 2;
   
   do until terminate == 1;
   
      Pt = TT*Pt*TT' + RR*QQ*RR';
      GAM0fin = GAM0fin+ l_bet^j*( ZZ*Pt*ZZ' + ZZ*RR*VV + (ZZ*RR*VV)' + HH );
      bet_sum = bet_sum + l_bet^j;
      polloss = l_vec'*GAM0fin*l_vec/bet_sum;
      
      if polloss > l_bound;
         polloss = l_bound;
         terminate = 1;
      elseif abs(polloss-polloss1) < l_crit;
         terminate = 1;
      endif;
      
      polloss1 = polloss;
      j = j+1;
      
   endo;

endif;

retp( polloss );

endp;
